# AutoSave.pl
# Auto-save v1.3b plugin for Movable Type
# http://www.chipple.net/mt/AutoSave/
# by Patrick Benny http://www.chipple.net/
# Free to use. All rights reserved.
# This code cannot be redistributed without permission from the author. 
#
# Version history
#   1.3.1 - 2007/5/24 - Fixed Gecko 1.9 bug (Firefox 3)
#   1.3a  - 2007/4/12 - Fixed Safari bug
#   1.2   - 2006/9/25 - Reset entry date/time when using "Save" button in bookmarklet
#                       "Saved at XX:XX:XX" now displays server time
#                       Fixed minor bug causing potential warning when using default settings (Thanks to drry)
#   1.1   - 2006/9/13 - Fixed bugs relating to bookmarklet
#   1.0.2 - 2006/9/12 - Added message in bookmarklet asking to select weblog
#   1.0.1 - 2006/9/11 - Fixed minor risk of auto-save after user clicked "Save"
#   1.0   - 2006/9/8  - First public release


# TODO: Add interval/timeout settings


use strict;
package MT::Plugin::AutoSave;

use vars qw( $VERSION );
$VERSION = '1.3.1';

use MT;
use MT::Plugin;


# Create plugin
my $plugin = MT::Plugin->new({
    name => "Auto-save",
    description => 'Auto-saves while editing a new or unpublished entry.',
    doc_link => 'http://www.chipple.net/mt/AutoSave/',
    author_name => 'Patrick Benny',
    author_link => 'http://www.chipple.net/',
    config_template => 'config.tmpl',
    settings => new MT::PluginSettings([
        ['interval', { Default => '60' }],
        ['timeout',  { Default => '30' }],
    ]),
    version => $VERSION,
});
MT->add_plugin($plugin);

## For Movable Type 3.3+
# Use AppTemplateSource to add auto-save script to edit_entry and bm_entry templates
MT->add_callback('MT::App::CMS::AppTemplateSource.edit_entry', 9, $plugin, \&_template);
MT->add_callback('MT::App::CMS::AppTemplateSource.bm_entry', 9, $plugin, \&_template);
# Use AppTemplateParam to add variables to edit_entry and bm_entry templates
MT->add_callback('MT::App::CMS::AppTemplateParam.edit_entry', 9, $plugin, \&_param);
MT->add_callback('MT::App::CMS::AppTemplateParam.bm_entry', 9, $plugin, \&_param);

## For Movable Type 3.2
# Use BigPAPI to add auto-save script to edit_entry template
MT->add_callback('bigpapi::template::edit_entry', 9, $plugin, \&_template);
MT->add_callback('bigpapi::template::bm_entry', 9, $plugin, \&_template);
# Use BigPAPI to add variables to edit_entry template
MT->add_callback('bigpapi::param::edit_entry', 9, $plugin, \&_param);
MT->add_callback('bigpapi::param::bm_entry', 9, $plugin, \&_param);


#################
### UTILITIES ###
#################

sub apply_default_settings {
    my $plugin = shift;
    my ($data, $scope) = @_;
    if ($scope ne 'system') {
        my $sys = $plugin->get_config_obj('system');
        my $sysdata = $sys->data();
        if ($plugin->{settings} && $sysdata) {
            foreach (keys %$sysdata) {
                $data->{$_} = $sysdata->{$_}
                    if (!exists $data->{$_}) || (!defined $data->{$_});
            }
        }
    } else {
        $plugin->SUPER::apply_default_settings(@_);
    }
}


################
### TEMPLATE ###
################

# Add auto-save script to edit_entry template
sub _template {

    my ($cb, $app, $template) = @_;

    my $cfg_obj = $plugin->get_config_obj('blog:' . $app->param('blog_id'));
    $cfg_obj ||= $plugin->get_config_obj('system');
    my ($interval, $timeout);
    if ($cfg_obj->id) {
      my $cfg_data = $cfg_obj->data();
      $interval = $cfg_data->{'interval'};
      $timeout = $cfg_data->{'timeout'};
    }
    else {
      $interval = $plugin->settings->defaults()->{'interval'};
      $timeout = $plugin->settings->defaults()->{'timeout'};
    }

    my $append;
    $append = '
<TMPL_IF NAME="USE_AUTO_SAVE">
<iframe name="autosave" style="width: 1px; height: 1px; visibility: hidden;"></iframe>
<script type="text/javascript">

var g_AS_form = document.forms["entry_form"];  // Entry form
// Keeping the real MT functions aside, these are replaced while the form is locked
var g_AS__clearDirty = self.clearDirty ? clearDirty : f_AS_DoNothing;  // MT\'s real clearDirty
var g_AS__submitForm = self.submitForm ? submitForm : f_AS_DoNothing;  // MT\'s real submitForm
var g_AS__onSubmit   = g_AS_form.onsubmit;                             // MT\'s real form onsubmit

// Auto-save function called every X seconds
function f_AS_AutoSave (bForce) {
  var f = g_AS_form;
  var els = f.elements;
  var el;
  // Don\'t auto-save if bookmarklet and if blog isn\'t selected yet
  if (  (el = els["blog_id"]).type == "select-one"
     && el.options[el.selectedIndex].value == "") {
    if (bForce)
      alert("<MT_TRANS phrase="You must choose a weblog in which to post the new entry." escape="singlequotes">"); 
    else
      f_AS_UpdateStatus("<span style=\"color: #900;\">Select weblog to enable auto-save</span>");
    return;
  }
  // If dirty, or if any of the main text fields has changed
  //   (setDirty only triggered on the onchange event)
  if (  bForce
     || (  (self.dirty && dirty)
        || ((el = els["title"]) && el.value.length != el.defaultValue.length)
        || ((el = els["text"]) && el.value.length != el.defaultValue.length)
        || ((el = els["text_more"]) && el.value.length != el.defaultValue.length)
        || ((el = els["excerpt"]) && el.value.length != el.defaultValue.length))) {
    // Lock form "Save" and "Preview" buttons
    f_AS_LockForm(true);
    // Set target and action to our script and hidden iframe
    var _target = f.target;
    var _action = f.action;
    f.target = "autosave";
    f.action = "'.$plugin->envelope.'/AutoSave.cgi";
<TMPL_IF NAME="IS_BM">
    // Pretend not being bookmarklet, in order to get the ID in the redirect URL
    var _is_bm = (el = els["is_bm"]).value;
    el.value = "";
</TMPL_IF>
    // clearDirty, and also copy the current values to defaultValue
    g_AS__clearDirty();
    if (el = els["title"]) el.defaultValue = el.value;
    if (el = els["text"]) el.defaultValue = el.value;
    if (el = els["text_more"]) el.defaultValue = el.value;
    if (el = els["excerpt"]) el.defaultValue = el.value;
    // Save silently
    f_AS_UpdateStatus("Saving...");
    g_AS__submitForm(f,"auto_save_entry");
    // Set safety unlock timer, in case the auto-save was to fail
    g_AS_oSafetyUnlock = setTimeout("f_AS_SafetyUnlock();",'.$timeout.'000);
    // Reset target and action
    f.target = _target;
<TMPL_IF NAME="IS_BM">
    els["__mode"].value = "bm_save_entry";
    els["is_bm"].value = _is_bm;
<TMPL_ELSE>
    f.action = _action;
</TMPL_IF>
    // NOTE: The form buttons remain locked until AutoSave.cgi returns
  }
}
function f_AS_AutoSaveForce () {
  f_AS_AutoSave(true);
}

// Lock form "Save" and "Preview" buttons while saving
function f_AS_LockForm (bLock) {
  var f = g_AS_form;
  f.onsubmit = bLock ? f_AS_OnSubmitLocked : g_AS__onSubmit;
  self.submitForm = bLock ? f_AS_SubmitFormLocked : f_AS_UserSubmitForm;
  self.clearDirty = bLock ? f_AS_DoNothing : g_AS__clearDirty;
  document.getElementById("as_savenow").onclick = bLock ? f_AS_SubmitFormLocked : f_AS_AutoSaveForce;
  // Clear safety unlock timer
  if (!bLock && g_AS_oSafetyUnlock) {
    clearTimeout(g_AS_oSafetyUnlock);
    g_AS_oSafetyUnlock = null;
  }
}
function f_AS_DoNothing () { }
function f_AS_OnSubmitLocked () { <TMPL_IF NAME="IS_BM">alert("Currently auto-saving, please wait.");</TMPL_IF> return false; }
function f_AS_SubmitFormLocked () { alert("Currently auto-saving, please wait."); }
function f_AS_UserSubmitForm (f,s) {
  if (g_AS_oTimer)
    clearTimeout(g_AS_oTimer);
  g_AS__submitForm(f,s);
}

// Start auto-save timer
var g_AS_oTimer = setInterval("f_AS_AutoSave();",'.$interval.'000);
if (self.submitForm) self.submitForm = f_AS_UserSubmitForm;

// Safety unlock timer
var g_AS_oSafetyUnlock = null;
function f_AS_SafetyUnlock () {
  g_AS_oSafetyUnlock = null;
  // Disable auto-save
  if (confirm("Auto-save could not complete within '.$timeout.' seconds.\nDisable for this session?") && g_AS_oTimer) {
    clearTimeout(g_AS_oTimer);
    g_AS_oTimer = null;
    f_AS_UpdateStatus("Auto-save disabled");
  }
  // Unlock form
  f_AS_LockForm(false);
}

function f_AS_UpdateStatus(sText) {
  document.getElementById("as_status").innerHTML = sText;
}

</script>
</TMPL_IF>
';

    $$template =~ s!(<TMPL_INCLUDE NAME="footer)!$append$1!;

    $append = '
<TMPL_UNLESS NAME="IS_BM"></div><div style="height: 0.8em;"></TMPL_UNLESS>
<small style="float: right;">
<TMPL_IF NAME="USE_AUTO_SAVE">
<span id="as_status"><TMPL_UNLESS NAME="IS_BM">Auto-save ready<TMPL_ELSE><span style="color: #900;">Select weblog to enable auto-save</span></TMPL_UNLESS></span> | <a href="javascript:void(0);" onclick="f_AS_AutoSaveForce();" id="as_savenow">Save now</a>
<TMPL_ELSE>
Auto-save unavailable (entry is not "Unpublished")
</TMPL_IF>
</small>
';

    $$template =~ s!(</textarea>)!$1$append!;

}

# Add author variables to edit_entry template
sub _param {

    my ($cb, $app, $param) = @_;

    if ($param->{id}) {
        my $entry = MT::Entry->load($param->{id});
        $param->{'use_auto_save'} = 1 if $entry->status == MT::Entry::HOLD();
    }
    else {
        $param->{'use_auto_save'} = 1;
    }

    $param->{'is_bm'} = 1 if $app->param('is_bm');
}